"""
Configs migration

Generation datetime: 2021-05-12 09:42:10.559163
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify

REVISION = "5da4c2f3"
DOWN_REVISION = "7777d7c8"


def monitoringSettingDowngradeFunction(setting: dict) -> dict:
    """
    Downgrade INFLUX_MONITORING to previous revision.
    """

    version = setting.pop("version")
    if version == 1:
        return setting

    # Try to migrate influx 2.0 setting (maybe the user has configured mappings and it will work)
    setting["database_name"] = setting.pop("bucket")
    setting["user_name"] = setting.pop("organization")
    setting["password"] = setting.pop("token")
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="INFLUX_MONITORING",
        oldValidationSchema={
            "properties": {
                "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                "use_ssl": {"type": "integer", "enum": [0, 1]},
                "flushing_period": {"type": "number"},
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "database_name": {"type": "string"},
                "user_name": {"type": "string"},
                "password": {"type": "string"},
            },
            "required": ["send_data_for_monitoring", "host", "port", "database_name"],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "description": "Monitoring settings.",
            "oneOf": [
                {
                    "properties": {
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "database_name": {"type": "string"},
                        "user_name": {"type": "string"},
                        "password": {"type": "string"},
                        "version": {"type": "integer", "enum": [1]},
                    },
                    "default": {
                        "version": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "user_name": "luna",
                        "password": "luna",
                    },
                    "required": [
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "database_name",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                },
                {
                    "properties": {
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                        "version": {"type": "integer", "enum": [2]},
                    },
                    "default": {"version": 2, "use_ssl": 0, "flushing_period": 1},
                    "required": [
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                },
            ],
        },
        oldDefaultValue={
            "send_data_for_monitoring": 0,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "database_name": "luna_monitoring",
            "user_name": "luna",
            "password": "luna",
        },
        newDefaultValue={
            "send_data_for_monitoring": 0,
            "version": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "database_name": "luna_monitoring",
            "user_name": "luna",
            "password": "luna",
        },
        settingUpgradeFunction=lambda setting: {**setting, "version": 1},
        settingDowngradeFunction=monitoringSettingDowngradeFunction,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-configurator",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-configurator",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
