"""
Module contains settings class for luna-lambda
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    ClusterCredentials as _ClusterCredentials,
    DBSetting as _DBSetting,
    LambdaBuildLimits as _LambdaBuildLimits,
    S3LambdaSettings as _S3LambdaSettings,
    ServiceAddressSettings as _ServiceAddressSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsLambda(BasePythonServiceSettingsClass):
    """
    Settings class for luna-lambda
    Attributes:
        db(DBSetting): database settings
        storageTime(str): storage time setting
        s3(_S3LambdaSettings): s3 lambda settings
        registry(str): registry
        insecureRegistries(list[str]): insecure registries
        clusterLocation (str): k8s cluster location
        clusterCredentials (ClusterCredentials): k8s cluster credentials
        licensesAddress(ServiceAddressSettings): luna-licenses address settings
    """

    serviceName = "luna-lambda"

    def __init__(self):
        super().__init__()
        self.db: Optional[_DBSetting] = None
        self.storageTime: Optional[str] = None
        self.s3: Optional[_S3LambdaSettings] = None
        self.registry: Optional[str] = None
        self.insecureRegistries: Optional[list[str]] = None
        self.clusterLocation: Optional[str] = None
        self.clusterCredentials: Optional[_ClusterCredentials] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.licensesAddress: Optional[_ServiceAddressSettings] = None
        self.buildLimits: Optional[_LambdaBuildLimits] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.db = _DBSetting(self.serviceName, self["LUNA_LAMBDA_DB"])
        self.storageTime = self["storage_time"]
        self.s3 = _S3LambdaSettings(self.serviceName, self["LAMBDA_S3"])
        self.registry = self["LAMBDA_REGISTRY"]
        self.insecureRegistries: list[str] = self["LAMBDA_INSECURE_REGISTRIES"]
        self.clusterLocation = self["CLUSTER_LOCATION"]
        if self.clusterLocation == "remote":
            self.clusterCredentials = _ClusterCredentials(self.serviceName, self["CLUSTER_CREDENTIALS"])
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.buildLimits = _LambdaBuildLimits(self.serviceName, self["LUNA_LAMBDA_BUILD_LIMITS"])
